import processing.core.*;

import java.util.*;

/**
 * A class for drawing 2D curves and handling mouse events
 *
 * @author Luca Castelli Aleardi (INF555, 2012)
 */
public class DrawCurve extends PApplet {
	
	Curve scheme; // interpolation scheme
	public int k=3; // default degree
	public int curveMode=0; // default interpolation scheme
	final static int curveTypes=3; // there are 3 types of curve interpolation
	
	LinkedList<Point_2> points=new LinkedList<Point_2>(); // the input points to interpolate
	final static double epsilon=4.;
	
	public Point_2 selectedPoint=null; // point selected with mouse events

	  public void setup() {
		  size(400,400); // size of the window
		  // choose the scheme and curves to draw
		  this.setInterpolationMethod();
	  }
	  
	  public void setInterpolationMethod() {		  
	      if(curveMode%curveTypes==0) {
	    	  this.scheme=new CubicBSpline(this, this.points);
	    	  this.k=3;
	    	  System.out.println("Cubic B-Splines");
	      }
	      else if(curveMode%curveTypes==1) {
	    	  this.scheme=new BSpline(this, this.points, this.k);
	    	  System.out.println("General B-Splines");
	      }
	      else {
	    	  System.out.println("No more schemes");
	      }
	      this.stroke(0, 0, 0);
	  }


	  public void draw() {
		  String msg=""+this.scheme.toString();
		  msg=msg+" - n="+this.points.size()+", k="+this.k;
	    background(220);
	    textFont(createFont("Arial",12,true),12);                 // STEP 4 Specify font to be used
	    fill(0);                        // STEP 5 Specify font color 
	    text(msg,10,this.height-20);  // STEP 6 Display Text
	    
	    if(points.isEmpty()) return; // no points to interpolate
	    scheme.plotCurve(1./1000.);
	    
	  }

	  public void removePoint(int x, int y) {
		  int index=findPoint(x, y);
		  if(index>=0 && index<this.points.size())
			  this.points.remove(index);
	  }
	  
	  public int findPoint(int x, int y) {
		  Point_2 p=new Point_2(x, y);
		  
		  int index=0;
		  boolean found=false;
		  for(Point_2 q:this.points) {
			  if(q.squareDistance(p)<epsilon) {
				  found=true;
				  break;
			  }
			  index++;
		  }
		  if(found==true)
			  return index;
		  else
			  return -1;
	  }

	  public Point_2 selectPoint(int x, int y) {
		  Point_2 p=new Point_2(x, y);
		  
		  int index=0;
		  boolean found=false;
		  for(Point_2 q:this.points) {
			  if(q.squareDistance(p)<epsilon) {
				  found=true;
				  break;
			  }
			  index++;
		  }
		  if(found==true)
			  return this.points.get(index);
		  else
			  return null;
	  }

	  public void mouseClicked() {
		  Point_2 p=new Point_2(mouseX, mouseY);
		  
		  if(mouseButton==LEFT && this.selectedPoint==null)
			  this.points.add(p);
		  else if(mouseButton==RIGHT)
			  removePoint(mouseX, mouseY);
		  
		  this.scheme.updateInputPoints(this.points);
	  }

	  public void mousePressed() {
		  this.selectedPoint=selectPoint(mouseX, mouseY);
	  }
	  
	  public void mouseReleased() {
		  
		  if(this.selectedPoint!=null) {
			  this.selectedPoint.setX(mouseX);
			  this.selectedPoint.setY(mouseY);
		  }
	  }
	  
	  public void mouseDragged() {
	  }
	  
	  public void keyPressed(){
		  switch(key) {
		  	case('s'):case('S'): { curveMode++; setInterpolationMethod(); } break;
		  	case('+'): { this.k++; setInterpolationMethod(); } break;
		  	case('-'): { this.k--; setInterpolationMethod(); } break;
		  }
	  }

}
